==> mingw-w64-czkawka-10.0.0-1.src.tar.zst: czkawka-10.0.0/czkawka_core/src/common/traits.rs <==
use std::fs::File;
use std::io::{BufWriter, Write};
use std::path::Path;
use std::sync::Arc;
use std::sync::atomic::AtomicBool;

use crossbeam_channel::Sender;
use fun_time::fun_time;
use serde::Serialize;

use crate::common::model::WorkContinueStatus;
use crate::common::progress_data::ProgressData;
use crate::common::tool_data::CommonData;

pub trait DebugPrint {
    fn debug_print(&self);
}

pub trait PrintResults {
    fn write_results<T: Write>(&self, writer: &mut T) -> std::io::Result<()>;

    #[fun_time(message = "print_results_to_output", level = "debug")]
    fn print_results_to_output(&self) {
        let stdout = std::io::stdout();
        let mut handle = stdout.lock();
        // Panics here are allowed, because it is used only in CLI
        self.write_results(&mut handle).expect("Error while writing to stdout");
        handle.flush().expect("Error while flushing stdout");
    }

    #[fun_time(message = "print_results_to_file", level = "debug")]
    fn print_results_to_file(&self, file_name: &str) -> std::io::Result<()> {
        let file_name: String = match file_name {
            "" => "results.txt".to_string(),
            k => k.to_string(),
        };

        let file_handler = File::create(file_name)?;
        let mut writer = BufWriter::new(file_handler);
        self.write_results(&mut writer)?;
        writer.flush()?;
        Ok(())
    }

    #[fun_time(message = "print_results_to_writer", level = "debug")]
    fn print_results_to_writer<T: Write>(&self, writer: &mut T) -> std::io::Result<()> {
        self.write_results(writer)
    }

    fn save_results_to_file_as_json(&self, file_name: &str, pretty_print: bool) -> std::io::Result<()>;

    fn save_results_to_file_as_json_internal<T: Serialize + std::fmt::Debug>(&self, file_name: &str, item_to_serialize: &T, pretty_print: bool) -> std::io::Result<()> {
        if pretty_print {
            self.save_results_to_file_as_json_pretty(file_name, item_to_serialize)
        } else {
            self.save_results_to_file_as_json_compact(file_name, item_to_serialize)
        }
    }

    #[fun_time(message = "save_results_to_file_as_json_pretty", level = "debug")]
    fn save_results_to_file_as_json_pretty<T: Serialize + std::fmt::Debug>(&self, file_name: &str, item_to_serialize: &T) -> std::io::Result<()> {
        let file_handler = File::create(file_name)?;
        let mut writer = BufWriter::new(file_handler);
        serde_json::to_writer_pretty(&mut writer, item_to_serialize)?;
        Ok(())
    }

    #[fun_time(message = "save_results_to_file_as_json_compact", level = "debug")]
    fn save_results_to_file_as_json_compact<T: Serialize + std::fmt::Debug>(&self, file_name: &str, item_to_serialize: &T) -> std::io::Result<()> {
        let file_handler = File::create(file_name)?;
        let mut writer = BufWriter::new(file_handler);
        serde_json::to_writer(&mut writer, item_to_serialize)?;
        Ok(())
    }

    fn save_all_in_one(&self, folder: &str, base_file_name: &str) -> std::io::Result<()> {
        let pretty_name = format!("{folder}/{base_file_name}_pretty.json");
        self.save_results_to_file_as_json(&pretty_name, true)?;
        let compact_name = format!("{folder}/{base_file_name}_compact.json");
        self.save_results_to_file_as_json(&compact_name, false)?;
        let txt_name = format!("{folder}/{base_file_name}.txt");
        self.print_results_to_file(&txt_name)?;
        Ok(())
    }
}

pub trait DeletingItems {
    #[must_use]
    fn delete_files(&mut self, stop_flag: &Arc<AtomicBool>, progress_sender: Option<&Sender<ProgressData>>) -> WorkContinueStatus;
}

pub trait ResultEntry {
    fn get_path(&self) -> &Path;
    fn get_modified_date(&self) -> u64;
    fn get_size(&self) -> u64;
}

pub trait Search {
    fn search(&mut self, stop_flag: &Arc<AtomicBool>, progress_sender: Option<&Sender<ProgressData>>);
}

pub trait AllTraits: DebugPrint + PrintResults + DeletingItems + CommonData + Search {}

==> mingw-w64-czkawka-11.0.0-1.src.tar.zst: czkawka-11.0.0/czkawka_core/src/common/traits.rs <==
use std::fs::File;
use std::io::{BufWriter, Write};
use std::path::Path;
use std::sync::Arc;
use std::sync::atomic::AtomicBool;

use crossbeam_channel::Sender;
use fun_time::fun_time;
use serde::Serialize;

use crate::common::model::WorkContinueStatus;
use crate::common::progress_data::ProgressData;
use crate::common::tool_data::CommonData;

pub trait DebugPrint {
    fn debug_print(&self);
}

pub trait PrintResults: CommonData {
    fn write_base_search_paths<T: Write>(&self, writer: &mut T) -> std::io::Result<()> {
        let dirs = &self.get_cd().directories;
        let included_paths = dirs.included_files.iter().chain(dirs.included_directories.iter()).collect::<Vec<_>>();
        let excluded_paths = dirs.excluded_files.iter().chain(dirs.excluded_directories.iter()).collect::<Vec<_>>();
        let reference_paths = dirs.reference_files.iter().chain(dirs.reference_directories.iter()).collect::<Vec<_>>();
        let excluded_items = self.get_cd().excluded_items.get_excluded_items();
        if self.get_cd().tool_type.may_use_reference_paths() {
            writeln!(
                writer,
                "Results of searching {included_paths:?} with reference paths {reference_paths:?}, excluded paths {excluded_paths:?} and excluded items {excluded_items:?}"
            )?;
            writeln!(
                writer,
                "(Before optimizations - included paths: {:?}, excluded paths: {:?}, reference paths: {:?})",
                dirs.original_included_paths, dirs.original_excluded_paths, dirs.original_reference_paths
            )?;
        } else {
            writeln!(
                writer,
                "Results of searching {included_paths:?} with excluded paths {excluded_paths:?} and excluded items {excluded_items:?}"
            )?;
            writeln!(
                writer,
                "(Before optimizations - included paths: {:?}, excluded paths: {:?})",
                dirs.original_included_paths, dirs.original_excluded_paths
            )?;
        }

        Ok(())
    }

    fn write_results<T: Write>(&self, writer: &mut T) -> std::io::Result<()>;

    #[fun_time(message = "print_results_to_output", level = "debug")]
    fn print_results_to_output(&self) {
        let stdout = std::io::stdout();
        let mut handle = stdout.lock();
        // Panics here are allowed, because it is used only in CLI
        self.write_results(&mut handle).expect("Error while writing to stdout");
        handle.flush().expect("Error while flushing stdout");
    }

    #[fun_time(message = "print_results_to_file", level = "debug")]
    fn print_results_to_file(&self, file_name: &str) -> std::io::Result<()> {
        let file_name: String = match file_name {
            "" => "results.txt".to_string(),
            k => k.to_string(),
        };

        let file_handler = File::create(file_name)?;
        let mut writer = BufWriter::new(file_handler);
        self.write_results(&mut writer)?;
        writer.flush()?;
        Ok(())
    }

    #[fun_time(message = "print_results_to_writer", level = "debug")]
    fn print_results_to_writer<T: Write>(&self, writer: &mut T) -> std::io::Result<()> {
        self.write_results(writer)
    }

    fn save_results_to_file_as_json(&self, file_name: &str, pretty_print: bool) -> std::io::Result<()>;

    fn save_results_to_file_as_json_internal<T: Serialize + std::fmt::Debug>(&self, file_name: &str, item_to_serialize: &T, pretty_print: bool) -> std::io::Result<()> {
        if pretty_print {
            self.save_results_to_file_as_json_pretty(file_name, item_to_serialize)
        } else {
            self.save_results_to_file_as_json_compact(file_name, item_to_serialize)
        }
    }

    #[fun_time(message = "save_results_to_file_as_json_pretty", level = "debug")]
    fn save_results_to_file_as_json_pretty<T: Serialize + std::fmt::Debug>(&self, file_name: &str, item_to_serialize: &T) -> std::io::Result<()> {
        let file_handler = File::create(file_name)?;
        let mut writer = BufWriter::new(file_handler);
        serde_json::to_writer_pretty(&mut writer, item_to_serialize)?;
        Ok(())
    }

    #[fun_time(message = "save_results_to_file_as_json_compact", level = "debug")]
    fn save_results_to_file_as_json_compact<T: Serialize + std::fmt::Debug>(&self, file_name: &str, item_to_serialize: &T) -> std::io::Result<()> {
        let file_handler = File::create(file_name)?;
        let mut writer = BufWriter::new(file_handler);
        serde_json::to_writer(&mut writer, item_to_serialize)?;
        Ok(())
    }

    fn save_all_in_one(&self, folder: &str, base_file_name: &str) -> std::io::Result<()> {
        let pretty_name = format!("{folder}/{base_file_name}_pretty.json");
        self.save_results_to_file_as_json(&pretty_name, true)?;
        let compact_name = format!("{folder}/{base_file_name}_compact.json");
        self.save_results_to_file_as_json(&compact_name, false)?;
        let txt_name = format!("{folder}/{base_file_name}.txt");
        self.print_results_to_file(&txt_name)?;
        Ok(())
    }
}

pub trait DeletingItems {
    #[must_use]
    fn delete_files(&mut self, stop_flag: &Arc<AtomicBool>, progress_sender: Option<&Sender<ProgressData>>) -> WorkContinueStatus;
}

pub trait FixingItems {
    type FixParams;

    fn fix_items(&mut self, stop_flag: &Arc<AtomicBool>, progress_sender: Option<&Sender<ProgressData>>, fix_params: Self::FixParams);
}

pub trait ResultEntry {
    fn get_path(&self) -> &Path;
    fn get_modified_date(&self) -> u64;
    fn get_size(&self) -> u64;
}

pub trait Search {
    fn search(&mut self, stop_flag: &Arc<AtomicBool>, progress_sender: Option<&Sender<ProgressData>>);
}

pub trait AllTraits: DebugPrint + PrintResults + DeletingItems + CommonData + Search {}
